from typing import Dict

import boto3
import rapidjson


class Bullhorn:
    def __init__(self, topic_map: Dict[str, str], client=None):
        if not client:
            client = boto3.client('sns')
        self._client = client
        self._topic_map = topic_map

    @classmethod
    def retrieve(cls, resource=None, profile=None):
        topic_map = {}
        if not resource:
            resource = boto3.resource('sns')
            if profile:
                resource = boto3.Session(profile_name=profile).resource('sns')
        topic_iterator = resource.topics.all()
        for entry in topic_iterator:
            attributes = entry.attributes
            topic_arn = attributes['TopicArn']
            display_name = attributes['DisplayName']
            if display_name:
                topic_map[display_name] = topic_arn
        client = boto3.client('sns')
        if profile:
            client = boto3.Session(profile_name=profile).client('sns')
        return cls(topic_map, client)

    @property
    def topic_map(self):
        return self._topic_map

    def publish_task(self,  task_name, flow_id, task_kwargs):
        msg = {
            'task_name': task_name,
            'flow_id': flow_id,
            'task_kwargs': task_kwargs
        }
        topic_arn = self._topic_map.get(task_name)
        if topic_arn is None:
            raise RuntimeError(f'there is no listener available for task_name: {task_name}')
        return self._publish('new_event', topic_arn, rapidjson.dumps(msg))

    def _publish(self, message_subject, topic_arn, message_body):
        response = self._client.publish(
            TopicArn=topic_arn,
            Subject=message_subject,
            Message=message_body
        )
        return response['MessageId']



import boto3
import pytest

from algernon.aws import Bullhorn


@pytest.mark.bullhorn
class TestBullhorn:
    def test_bullhorn_retrieve(self):
        bullhorn = Bullhorn.retrieve(profile='dev')
        assert bullhorn



import rapidjson


def queued(production_fn):
    def wrapper(*args):
        results = []
        event = args[0]
        context = args[1]
        for entry in event['Records']:
            entry_body = rapidjson.loads(entry['body'])
            original_payload = rapidjson.loads(entry_body['Message'])
            results.append(production_fn(original_payload, context))
        return results
    return wrapper



import setuptools

setuptools.setup(
    name='moncrief',
    version='2.1.0',
    url='https://github.com/AlgernonSolutions/algernon',
    license='GNU Affero General Public License v3.0',
    author='algernon_solutions/jcubeta',
    author_email='jcubeta@algernon.solutions',
    description='This library contains the basic units of functionality and infrastructure needed to effectively run '
                'operations and applications in a distributed and severless fashion.',
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'aws-xray-sdk',
        'boto3',
        'botocore',
        'certifi',
        'chardet',
        'docutils',
        'future',
        'idna',
        'jmespath',
        'jsonpickle',
        'jsonref',
        'python-dateutil',
        'python-rapidjson',
        'pytz',
        'requests',
        's3transfer',
        'six',
        'urllib3',
        'wrapt'
    ],
)
